/* --COPYRIGHT--,BSD
 * Copyright (c) 2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//*****************************************************************************

/*
 * LCDSetup.c
 *
 *The Following Code sets up the MSP430 to use the LCD
 *
 */
#include <msp430.h>
#include <DRVGlobalVariables.h>

char d0 = 10;                                                        //Extracted number that will be displayed 0 digit place, (10 = blank bit pattern)
char d1 = 10;                                                        //Extracted number that will be displayed 1 digit place
char d2 = 10;                                                        //Extracted number that will be displayed 2 digit place
char d3 = 10;                                                        //Extracted number that will be displayed 3 digit place

void LCDSetup()
{
        SYSCFG2 |= LCDPCTL;                                                //
        LCDPCTL0 |= 0xFF7F;                                                //L0-L5 and L8-L15 pins selected for LCD function//+L6
        LCDPCTL1 |= 0xFFFF;                                                //L16-L21 and L24-L31 pins selected for LCD function//+L22,L23
        LCDPCTL2 |= 0x0004;                                                //L32,L33,L36,L37 pins selected for LCD function//only L34
        LCDCTL0 = LCDSSEL_1 | LCDDIV_7;                                    //fLCD ref=fsource=ACLK=32768Hz, fLCD=fsource/(LCDDIVx+1)*value[LCDMXx]
                                                                           //fLCD = 32768/((7+1)*64)=64Hz;
                                                                           //f_Charge_pump = 256Hz, Internal Vref = 2.96V, Mode 3
        LCDVCTL = LCDCPEN | LCDREFEN | VLCD_6 | (LCDCPFSEL0 | LCDCPFSEL1 | LCDCPFSEL2 | LCDCPFSEL3);
        LCDMEMCTL |= LCDCLRM;                                              //Clear LCD memory
        LCDCSSEL0 = 0x0000;                                                //L5 COM pin, all others Segments
        LCDCSSEL1 = BIT0;                                                  //pins L16-L31 set to be segments
        LCDCSSEL2 = 0x0000;                                                //pins L32-L47 set to be segments
        LCDM8|=0x01;                                                       //L16 = COM    OLD://L5 = COM0


        LCDCTL0=0;                                                         //Clear this register to remove any previous default values
        LCDCTL0 |= LCDDIV_13 | LCDSSEL_1 | LCDSTATIC | LCDSON | LCDON;     //Divide by 13, clock source =ACLK, LCD Mux =000b (Static), enable all LCD segments, turn LCD on
}
void dig1(num)
{
    switch(num)
    {
    case 0 : LCDM0=0x11; LCDM1&=~0x01; LCDM8|= 0x10; LCDM9=0x11; LCDM10|= 0x01; break;    //"_ _ _ 0"
    case 1 : LCDM0=0x00; LCDM1&=~0x01; LCDM8|= 0x10; LCDM9=0x01; LCDM10&=~0x01; break;    //"_ _ _ 1"
    case 2 : LCDM0=0x01; LCDM1|= 0x01; LCDM8|= 0x10; LCDM9=0x10; LCDM10|= 0x01; break;    //"_ _ _ 2"
    case 3 : LCDM0=0x01; LCDM1|= 0x01; LCDM8|= 0x10; LCDM9=0x11; LCDM10&=~0x01; break;    //"_ _ _ 3"
    case 4 : LCDM0=0x10; LCDM1|= 0x01; LCDM8|= 0x10; LCDM9=0x01; LCDM10&=~0x01; break;    //"_ _ _ 4"
    case 5 : LCDM0=0x11; LCDM1|= 0x01; LCDM8&=~0x10; LCDM9=0x11; LCDM10&=~0x01; break;    //"_ _ _ 5"
    case 6 : LCDM0=0x11; LCDM1|= 0x01; LCDM8&=~0x10; LCDM9=0x11; LCDM10|= 0x01; break;    //"_ _ _ 6"
    case 7 : LCDM0=0x01; LCDM1&=~0x01; LCDM8|= 0x10; LCDM9=0x01; LCDM10&=~0x01; break;    //"_ _ _ 7"
    case 8 : LCDM0=0x11; LCDM1|= 0x01; LCDM8|= 0x10; LCDM9=0x11; LCDM10|= 0x01; break;    //"_ _ _ 8"
    case 9 : LCDM0=0x11; LCDM1|= 0x01; LCDM8|= 0x10; LCDM9=0x11; LCDM10&=~0x01; break;    //"_ _ _ 9"
    case 10: LCDM0=0x00; LCDM1&=~0x01; LCDM8&=~0x10; LCDM9=0x00; LCDM10&=~0x01; break;    //"_ _ _  "
    case 11: LCDM0=0x00; LCDM1|= 0x01; LCDM8&=~0x10; LCDM9=0x00; LCDM10&=~0x01;           //"_ _ _ -"
    }
}
void dig2(num)
{
    switch(num)
    {
    case 0 : LCDM1|= 0x10; LCDM2=0x11; LCDM3&=~0x01; LCDM11=0x11; LCDM12|= 0x01; break;    //"_ _ 0 _"
    case 1 : LCDM1|= 0x10; LCDM2=0x00; LCDM3&=~0x01; LCDM11=0x01; LCDM12&=~0x01; break;    //"_ _ 1 _"
    case 2 : LCDM1|= 0x10; LCDM2=0x01; LCDM3|= 0x01; LCDM11=0x10; LCDM12|= 0x01; break;    //"_ _ 2 _"
    case 3 : LCDM1|= 0x10; LCDM2=0x01; LCDM3|= 0x01; LCDM11=0x11; LCDM12&=~0x01; break;    //"_ _ 3 _"
    case 4 : LCDM1|= 0x10; LCDM2=0x10; LCDM3|= 0x01; LCDM11=0x01; LCDM12&=~0x01; break;    //"_ _ 4 _"
    case 5 : LCDM1&=~0x10; LCDM2=0x11; LCDM3|= 0x01; LCDM11=0x11; LCDM12&=~0x01; break;    //"_ _ 5 _"
    case 6 : LCDM1&=~0x10; LCDM2=0x11; LCDM3|= 0x01; LCDM11=0x11; LCDM12|= 0x01; break;    //"_ _ 6 _"
    case 7 : LCDM1|= 0x10; LCDM2=0x01; LCDM3&=~0x01; LCDM11=0x01; LCDM12&=~0x01; break;    //"_ _ 7 _"
    case 8 : LCDM1|= 0x10; LCDM2=0x11; LCDM3|= 0x01; LCDM11=0x11; LCDM12|= 0x01; break;    //"_ _ 8 _"
    case 9 : LCDM1|= 0x10; LCDM2=0x11; LCDM3|= 0x01; LCDM11=0x11; LCDM12&=~0x01; break;    //"_ _ 9 _"
    case 10: LCDM1&=~0x10; LCDM2=0x00; LCDM3&=~0x01; LCDM11=0x00; LCDM12&=~0x01; break;    //"_ _   _"
    case 11: LCDM1&=~0x10; LCDM2=0x00; LCDM3|= 0x01; LCDM11=0x00; LCDM12&=~0x01;           //"_ _ - _"
    }
}
void dig3(num)
{
    switch(num)
    {
    case 0 : LCDM4=0x11; LCDM5=0x01; LCDM13=0x11; LCDM14|= 0x01; break;    //"_ 0 _ _"
    case 1 : LCDM4=0x01; LCDM5=0x00; LCDM13=0x01; LCDM14&=~0x01; break;    //"_ 1 _ _"
    case 2 : LCDM4=0x11; LCDM5=0x10; LCDM13=0x10; LCDM14|= 0x01; break;    //"_ 2 _ _"
    case 3 : LCDM4=0x11; LCDM5=0x10; LCDM13=0x11; LCDM14&=~0x01; break;    //"_ 3 _ _"
    case 4 : LCDM4=0x01; LCDM5=0x11; LCDM13=0x01; LCDM14&=~0x01; break;    //"_ 4 _ _"
    case 5 : LCDM4=0x10; LCDM5=0x11; LCDM13=0x11; LCDM14&=~0x01; break;    //"_ 5 _ _"
    case 6 : LCDM4=0x10; LCDM5=0x11; LCDM13=0x11; LCDM14|= 0x01; break;    //"_ 6 _ _"
    case 7 : LCDM4=0x11; LCDM5=0x00; LCDM13=0x01; LCDM14&=~0x01; break;    //"_ 7 _ _"
    case 8 : LCDM4=0x11; LCDM5=0x11; LCDM13=0x11; LCDM14|= 0x01; break;    //"_ 8 _ _"
    case 9 : LCDM4=0x11; LCDM5=0x11; LCDM13=0x11; LCDM14&=~0x01; break;    //"_ 9 _ _"
    case 10: LCDM4=0x00; LCDM5=0x00; LCDM13=0x00; LCDM14&=~0x01; break;    //"_   _ _"
    case 11: LCDM4=0x00; LCDM5=0x10; LCDM13=0x00; LCDM14&=~0x01;           //"_ - _ _"
    }
}
void dig4(num)
{
    switch(num)
    {
    case 0 : LCDM6=0x11; LCDM7=0x01; LCDM15=0x11; LCDM17|= 0x01; break;    //"0 _ _ _"
    case 1 : LCDM6=0x01; LCDM7=0x00; LCDM15=0x01; LCDM17&=~0x01; break;    //"1 _ _ _"
    case 2 : LCDM6=0x11; LCDM7=0x10; LCDM15=0x10; LCDM17|= 0x01; break;    //"2 _ _ _"
    case 3 : LCDM6=0x11; LCDM7=0x10; LCDM15=0x11; LCDM17&=~0x01; break;    //"3 _ _ _"
    case 4 : LCDM6=0x01; LCDM7=0x11; LCDM15=0x01; LCDM17&=~0x01; break;    //"4 _ _ _"
    case 5 : LCDM6=0x10; LCDM7=0x11; LCDM15=0x11; LCDM17&=~0x01; break;    //"5 _ _ _"
    case 6 : LCDM6=0x10; LCDM7=0x11; LCDM15=0x11; LCDM17|= 0x01; break;    //"6 _ _ _"
    case 7 : LCDM6=0x11; LCDM7=0x00; LCDM15=0x01; LCDM17&=~0x01; break;    //"7 _ _ _"
    case 8 : LCDM6=0x11; LCDM7=0x11; LCDM15=0x11; LCDM17|= 0x01; break;    //"8 _ _ _"
    case 9 : LCDM6=0x11; LCDM7=0x11; LCDM15=0x11; LCDM17&=~0x01; break;    //"9 _ _ _"
    case 10: LCDM6=0x00; LCDM7=0x00; LCDM15=0x00; LCDM17&=~0x01; break;    //"  _ _ _"
    case 11: LCDM6=0x00; LCDM7=0x10; LCDM15=0x00; LCDM17&=~0x01;           //"- _ _ _"
    }
}
void decimal(num)
{
    switch(num)
    {
    case 1 : LCDM10|= 0x10; LCDM12&=~0x10; LCDM14&=~0x10; break;    //"_ _ _._"
    case 2 : LCDM10&=~0x10; LCDM12|= 0x10; LCDM14&=~0x10; break;    //"_ _._ _"
    case 3 : LCDM10&=~0x10; LCDM12&=~0x10; LCDM14|= 0x10; break;    //"_._ _ _"
    case 4 : LCDM10&=~0x10; LCDM12&=~0x10; LCDM14&=~0x10;           //"_ _ _ _"
    }
}

void sel()
{
    LCDM6=0x00; LCDM7=0x00; LCDM15=0x00; LCDM17&=~0x01;                    //"  _ _ _"
    LCDM4=0x10; LCDM5=0x11; LCDM13=0x11; LCDM14&=~0x01;                    //"_ S _ _"
    LCDM1&=~0x10; LCDM2=0x11; LCDM3|= 0x01; LCDM11=0x10; LCDM12|= 0x01;    //"_ _ E _"
    LCDM0=0x10; LCDM1&=~0x01; LCDM8&=~0x10; LCDM9=0x10; LCDM10|= 0x01;     //"_ _ _ L"
    decimal(4);
}

void spin()
{
    LCDM6=0x10; LCDM7=0x11; LCDM15=0x11; LCDM17&=~0x01;                    //"S _ _ _"
    LCDM4=0x11; LCDM5=0x11; LCDM13=0x00; LCDM14|= 0x01;                    //"_ P _ _"
    LCDM1|= 0x10; LCDM2=0x00; LCDM3&=~0x01; LCDM11=0x01; LCDM12&=~0x01;    //"_ _ I _"
    LCDM0=0x00; LCDM1|= 0x01; LCDM8&=~0x10; LCDM9=0x01; LCDM10|= 0x01;     //"_ _ _ n"
    decimal(4);
}

void deg()
{
    LCDM0=0x11; LCDM1|= 0x01; LCDM8|= 0x10; LCDM9=0x00; LCDM10&=~0x01;    //"_ _ _ "
}
void cp()
{
    LCDM6=0x10; LCDM7=0x01; LCDM15=0x10; LCDM17|= 0x01;    //"C _ _ _"
    LCDM4=0x11; LCDM5=0x11; LCDM13=0x00; LCDM14|= 0x01;    //"_ P _ _"
}
//Function choosing what number to write to all 4 digits when ADC > 999
void d4w(c_val)
{
    d3=c_val/1000;                                          //d3 x x x ==> 0 0 0 d3
    d2=(c_val-d3*1000)/100;                                 //(d3 d2 x x - d3 0 0 0)/100  ==> 0 0 0 d2
    d1=(c_val-d3*1000-d2*100)/10;                           //(d3 d2 d1 x - d3 d2 0 0)/10  ==> 0 0 0 d1
    d0=(c_val-d3*1000-d2*100-d1*10);                        //(d3 d2 d1 d0 - d3 d2 d1 0)  ==> 0 0 0 d0
}
//Function choosing what number to write to all 4 digits when 999>ADC>99
void d3w(c_val)
{
    if(dispMode == 4){
        d3=11;                                                  // Leading 0, change to 10 for blank screen
    }else{
        d3=10;
    }
    d2=(c_val)/100;                                         // d2 x x ==> 0 0 0 d2
    d1=(c_val-d2*100)/10;                                   // (d2 d1 x - d2 0 0)/10  ==> 0 0 d1
    d0=(c_val-d2*100-d1*10);                                // (d2 d1 d0 - d2 d1 0)  ==> 0 0 d0
}
//Function choosing what number to write to all 4 digits when 99>ADC>9
void d2w(c_val)
{
    d3=10;                                                  // 0 if blank screen not desired
    if(dispMode == 4){
        d2=11;                                                  // 10 if blank screen desired
    }else{
        d2=10;
    }
    d1=(c_val)/10;
    d0=(c_val-d1*10);
}
//Function choosing what number to write to all 4 digits when 9>ADC
void d1w(c_val)
{
    d3=10;                                                  // set to 0 if blank screen not desired
    if(dispMode == 4){
        d2=11;                                                  // set to 0 if blank screen not desired
    }else{
        d2=10;
    }
    if(dispMode == 1 || dispMode == 4){
        d1=0;                                               // set to 10 if blank screen desired
    }else{
        d1=10;
    }
    d0=c_val;
}

void LCDOUT(int desplayINT)
{
    if(dispMode==1)                                         //dispMode1=Degree with one decimal LCD display
    {
        if (desplayINT > 999) {d4w(desplayINT);}            //if ADC memory value greater than 999, call d4w method
        else if (desplayINT >99){d3w(desplayINT);}          //if ADC memory value greater than 99, call d3w method
        else if (desplayINT > 9){d2w(desplayINT);}          //if ADC memory value greater than 9, call d2w method
        else                   {d1w(desplayINT);}           //if ADC memory value less than 9, call d1w method
        dig1(d0); dig2(d1); dig3(d2); dig4(d3); decimal(1); //Convert d0-d3 to correct bit pattern for LCD
    }
    else if(dispMode==2)                                    //dispMode2 = calibration point followed by number. ex: CP. 8
    {
        if (desplayINT > 999) {d4w(desplayINT);}            //if ADC memory value greater than 999, call d4w method
        else if (desplayINT >99){d3w(desplayINT);}          //if ADC memory value greater than 99, call d3w method
        else if (desplayINT > 9){d2w(desplayINT);}          //if ADC memory value greater than 9, call d2w method
        else                   {d1w(desplayINT);}           //if ADC memory value less than 9, call d1w method
        dig1(d0); dig2(d1); cp(); decimal(2);               //Convert d0-d3 to correct bit pattern for LCD
    }
    else if(dispMode==3)                                    //dispMode3 = Degree with sign LCD display
    {
        if (desplayINT > 999) {d4w(desplayINT);}            //if ADC memory value greater than 999, call d4w method
        else if (desplayINT >99){d3w(desplayINT);}          //if ADC memory value greater than 99, call d3w method
        else if (desplayINT > 9){d2w(desplayINT);}          //if ADC memory value greater than 9, call d2w method
        else                   {d1w(desplayINT);}           //if ADC memory value less than 9, call d1w method
        dig2(d0); dig3(d1); dig4(d2); deg(); decimal(4);    //Convert d0-d3 to correct bit pattern for LCD
    }
    else if(dispMode==4)                                         //dispMode1=Degree with one decimal LCD display
    {
        if (desplayINT > 999) {d4w(desplayINT);}            //if ADC memory value greater than 999, call d4w method
        else if (desplayINT >99){d3w(desplayINT);}          //if ADC memory value greater than 99, call d3w method
        else if (desplayINT > 9){d2w(desplayINT);}          //if ADC memory value greater than 9, call d2w method
        else                   {d1w(desplayINT);}           //if ADC memory value less than 9, call d1w method
        dig1(d0); dig2(d1); dig3(d2); dig4(d3); decimal(1); //Convert d0-d3 to correct bit pattern for LCD
    }
}
